﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;
using System.ComponentModel;
using System.Threading;

namespace GarbageTest {
  class Program {
    const int K = 100;
    const int N = 400000;
    
    static void Main(string[] args) {
      for (int k = 0; k < K; k++) {
        Test();
      }
    }

    public static void Test() {
      HiPerfTimer counter = new HiPerfTimer();
      counter.Start();
      CreateObjects();
      counter.Stop();
      Console.WriteLine(string.Format("{0}\t{1}\t{2}\t{3}", counter.Duration, GC.CollectionCount(0), GC.CollectionCount(1), GC.CollectionCount(2)));
    }

    public static TestObject last = null;

    private static void CreateObjects() {
      for (int i = 0; i < N; i++) {
        last = new TestObject(last);
      }
    }
  }

  public class TestObject {
    public TestObject previous;

    public TestObject(TestObject p) { previous = p; }
  }

  internal class HiPerfTimer {
    [DllImport("Kernel32.dll")]
    private static extern bool QueryPerformanceCounter(
        out long lpPerformanceCount);

    [DllImport("Kernel32.dll")]
    private static extern bool QueryPerformanceFrequency(
        out long lpFrequency);

    private long startTime, stopTime;
    private long freq;

    // Constructor
    public HiPerfTimer() {
      startTime = 0;
      stopTime = 0;

      if (QueryPerformanceFrequency(out freq) == false) {
        // high-performance counter not supported
        throw new Win32Exception();
      }
    }

    // Start the timer
    public void Start() {
      QueryPerformanceCounter(out startTime);
    }

    // Stop the timer
    public void Stop() {
      QueryPerformanceCounter(out stopTime);
    }

    // Returns the duration of the timer (in seconds)
    public double Duration {
      get {
        return (double)(stopTime - startTime) / (double)freq;
      }
    }
  }
}
