using System;
using System.Threading;

namespace TokenRing
{
  class Node 
  {
    Node next;
    Thread t;
    bool hasToken;
    
    public Node() {
      next = null;
      hasToken = false;
    }

    public void SetNeighbour(Node nextNode) 
    {
      next = nextNode;
    }

    public void Start() 
    {
      t = new Thread(new ThreadStart(Run));
      t.Start();
    }

    void Run() {
      lock(this) 
      {
        for(int k = 1; k <= TokenRing.K; k++) { 
          while (!hasToken) { Monitor.Wait(this); }
          hasToken = false;
          next.PassToken();
          Monitor.PulseAll(this);
        }
      }
    }

    public void PassToken() 
    {
      lock(this) 
      {
        while (hasToken) { Monitor.Wait(this); }
        hasToken = true; 
        Monitor.PulseAll(this);
      }
    }

    public void Terminate() 
    {
      t.Join();
    }
  }

	class TokenRing
	{
    public const int N = 1000;
    public const int K = 1000;
    
		static void Main(string[] args)
		{
      DateTime startTime = DateTime.Now;
      Node[] node = new Node[N];
      for (int i = 0; i < N; i++) 
      {
        node[i] = new Node();
      }
      for (int i = 0; i < N; i++) 
      {
        node[i].SetNeighbour(node[(i+1) % N]);
        node[i].Start();
      }
      node[0].PassToken();
      for (int i = 0; i < N; i++) 
      {
        node[i].Terminate();
      }
      Console.WriteLine((DateTime.Now-startTime) + " ms");
		}
	}
}
