abstract class NumberStream {
  public abstract boolean IsFinished();
  public abstract int Get();
}

class Sieve extends NumberStream implements Runnable {
  NumberStream source;
  Thread t;
  int prime;
  boolean hasPrime;
  int next;
  boolean hasNext;
  boolean finished;

  public Sieve(NumberStream prev) {
    source = prev;
    hasPrime = false; hasNext = false;
    finished = false;
    t = new Thread(this);
    t.start();
  }
  
  public void run() {
    if (!source.IsFinished()) {
      synchronized(this) { 
        prime = source.Get(); hasPrime = true; 
        notifyAll(); 
      }
    }
    while (!source.IsFinished()) {
      synchronized(this) {
        int i = source.Get();
        if (i % prime != 0) {
          while (hasNext) {
            try { wait(); }
            catch(InterruptedException e) { throw new RuntimeException(""); }
          }
          next = i; hasNext = true;
          notifyAll();
        }
      }
    }
    synchronized(this) { 
      finished = true; notifyAll(); 
    }
  }

  public boolean HasPrime() {
    synchronized(this) {
      while (!hasPrime && !finished) { 
        try { wait(); }
        catch(InterruptedException e) { throw new RuntimeException(""); }
      }
    }
    return hasPrime;  
  }
  
  public int GetPrime() {
    return prime;
  }
  
  public boolean IsFinished() {
    synchronized(this) {
      while (!hasNext && !finished) {
        try { wait(); }
        catch(InterruptedException e) { throw new RuntimeException(""); }
      }
      return !hasNext;
    }
  }

  public int Get() {
    synchronized(this) {
      while (!hasNext) { 
        try { wait(); }
        catch(InterruptedException e) { throw new RuntimeException(""); }      
      }
      int x = next; hasNext = false;
      notifyAll();
      return x;
    }
  }
}

class NumberGenerator extends NumberStream implements Runnable {
  public static final int N = 10000;
  public static final int None = -1;
  Thread t;
  int next;
  boolean hasNext;
  boolean finished;

  public NumberGenerator() {
    hasNext = false;
    finished = false;
    t = new Thread(this);
    t.start();
  }

  public void run() {
    for (int i = 2; i <= N; i++) {
      synchronized(this) {
        while (hasNext) { 
          try { wait(); }
          catch(InterruptedException e) { throw new RuntimeException(""); }
        }
        next = i; hasNext = true;
        notifyAll();
      }
    }
    synchronized(this) {
      finished = true;
      notifyAll();
    }
  }

  public boolean IsFinished() {
    synchronized(this) {
      while (!hasNext && !finished) { 
        try { wait(); }
        catch(InterruptedException e) { throw new RuntimeException(""); }      
      }
      return !hasNext;
    }
  }

  public int Get() {
    synchronized(this) {
      while (!hasNext) { 
        try { wait(); }
        catch(InterruptedException e) { throw new RuntimeException(""); }      
      }
      int x = next; hasNext = false;
      notifyAll();
      return x;
    }
  }
}

class Eratosthenes
{
	public static void main(String[] args)
	{
    long startTime = System.currentTimeMillis();
    NumberGenerator generator = new NumberGenerator();
    int i = 1;
    Sieve sieve = new Sieve(generator);
    while (sieve.HasPrime()) {
      int p = sieve.GetPrime();
      //System.out.print(p + " ");
      sieve = new Sieve(sieve); i++;
    }
    //System.out.println();
    System.out.println((System.currentTimeMillis() - startTime) + " ms");
  }
}
