class BoundedBuffer {
  final static int C = 10;
  int[] a;
  int first, last;
  int nofProducers;
  
  public BoundedBuffer(int n) {
    a = new int[C];
    first = 0; last = 0; nofProducers = n;
  }

  public void Put(int x) {
    synchronized(this) {
      while (last-first >= C) { 
        try { wait(); }
        catch(InterruptedException e) { throw new RuntimeException(); }
      }
      a[last % C] = x; last++;
      notifyAll();
    }
  }

  public boolean InputAvailable() {
    synchronized(this) {
      while ((first >= last) && (nofProducers > 0)) { 
        try { wait(); }
        catch(InterruptedException e) { throw new RuntimeException(); }
      }
      return first < last;
    }
  }
  
  public int Get() {
    synchronized(this) {
      int x = a[first % C]; first++; 
      notifyAll();
      return x;
    }
  }

  public void Finish() {
    synchronized(this) { 
      nofProducers--;
      if (nofProducers == 0) { notifyAll(); }
    }
  }
}

class Producer implements Runnable {
  BoundedBuffer buffer;
  Thread t;

  public Producer(BoundedBuffer b) {
    buffer = b;
    t = new Thread(this);
  }

  public void Start() {
    t.start();
  }

  public void run() {
    final int K = 1000000;
    for (int i = 1; i <= K; i++) {
      buffer.Put(i);
    }
    buffer.Finish();
  }

  public void Terminate() {
    try { t.join(); }
    catch(InterruptedException e) { throw new RuntimeException(); }
  }
}

class Consumer implements Runnable {
  BoundedBuffer buffer;
  Thread t;

  public Consumer(BoundedBuffer b) {
    buffer = b;
    t = new Thread(this);
  }
  
  public void Start() {
    t.start();
  }

  public void run() {
    while(buffer.InputAvailable()) {
      int x = buffer.Get();
      //if (x % 100000 == 0) { System.out.println(x); }
    }
  }

  public void Terminate() {
    try { t.join(); }
    catch(InterruptedException e) { throw new RuntimeException(); }
  }
}

public class ProducerConsumer
{
  public static void main(String[] args) {
    final int N = 5;
    final int M = 5;

    long startTime = System.currentTimeMillis();
    BoundedBuffer buffer = new BoundedBuffer(N);
    Producer[] producer = new Producer[N];
    Consumer[] consumer = new Consumer[M];
    for (int i = 0; i < N; i++) { producer[i] = new Producer(buffer); }
    for (int i = 0; i < M; i++) { consumer[i] = new Consumer(buffer); }
    for (int i = 0; i < N; i++) { producer[i].Start(); }
    for (int i = 0; i < M; i++) { consumer[i].Start(); }
    for (int i = 0; i < N; i++) { producer[i].Terminate(); }
    for (int i = 0; i < M; i++) { consumer[i].Terminate(); }
    System.out.println((System.currentTimeMillis() - startTime) + " ms");
  }
}

