class Node implements Runnable 
{
  Node next;
  Thread t;
  boolean hasToken;
  
  public Node() {
    next = null;
    hasToken = false;
  }

  public void SetNeighbour(Node nextNode) 
  {
    next = nextNode;
  }

  public void Start() 
  {
    t = new Thread(this);
    t.start();
  }

  public void run() {
    synchronized(this) 
    {
      for(int k = 1; k <= TokenRing.K; k++) { 
        while (!hasToken) { 
          try { wait(); }
          catch(InterruptedException e) { throw new RuntimeException(""); }
        }
        hasToken = false;
        next.PassToken();
        notifyAll();
      }
    }
  }

  public void PassToken() 
  {
    synchronized(this) 
    {
      while (hasToken) { 
        try { wait(); }
        catch(InterruptedException e) { throw new RuntimeException(""); }
      }
      hasToken = true; 
      notifyAll();
    }
  }

  public void Terminate() 
  {
    try { t.join(); }
    catch(InterruptedException e) { throw new RuntimeException(""); }
  }
}

class TokenRing
{
  public static final int N = 1000;
  public static final int K = 1000;
  
	public static void main(String[] args)
	{
    long startTime = System.currentTimeMillis();
    Node[] node = new Node[N];
    for (int i = 0; i < N; i++) 
    {
      node[i] = new Node();
    }
    for (int i = 0; i < N; i++) 
    {
      node[i].SetNeighbour(node[(i+1) % N]);
      node[i].Start();
    }
    node[0].PassToken();
    for (int i = 0; i < N; i++) 
    {
      node[i].Terminate();
    }
    System.out.println((System.currentTimeMillis()-startTime) + " ms");
	}
}
